### Script to analyze spider evolution
### Jonas Wolff & Kawsar Khan
### Last update: 17/04/2023

#packages
library(phytools)
library(phylolm)
library(geiger)
library(sensiPhy)
library(rlang)
library(ggplot2)
library(vioplot)
library(Durga)

rm(list = ls())


#load data
p <- read.nexus("input/50-genes-2022-06-30-annotated-CAH.trees")
p <- drop.tip(p, tip="Matachiinae_spec_4_DES102") #tip with no trait data
# remove outgroup for comparative analysis
p <- drop.tip(p, tip=c("Helpis_minitabunda_KWS603", "Isopeda_villosa_MK001", "Nosterella_sp_CYC010"))
p <- force.ultrametric(p) # necessary to correct numeric non-ultrametricity after tip drop
p <- ladderize(p)

# tree with alternative topology (position of Agelenidae)
p2 <- read.nexus("input/50-genes-2022-06-30-Agelenidae-constraint-annotated-CAH.trees")
p2 <- drop.tip(p2, tip="Matachiinae_spec_4_DES102")
# remove outgroup for comparative analysis
p2 <- drop.tip(p2, tip=c("Helpis_minitabunda_KWS603", "Isopeda_villosa_MK001", "Nosterella_sp_CYC010"))
p2 <- force.ultrametric(p2) # necessary to correct numeric non-ultrametricity after tip drop
p2 <- ladderize(p2)

d <- read.csv("input/spider_species_trait_list.csv")
rownames(d) <- d$terminal

name.check(p, d)

plotTree(p, fsize=.5, lwd=1)
plotTree(p2, fsize=.5, lwd=1)

# for analysis with p2 use:
# p <- p2

##------ Ecology --------##
# web building
dat <- match_dataphy(web ~ 1, data = d, phy = p)
web <- set_names(dat$data[, "web"], rownames(dat$data))
# first, we test the fit of alternative unconstrained evolutionary models
fit_ER <- fitDiscrete(p, web, model = "ER") 
fit_ARD <- fitDiscrete(p, web, model = "ARD")  
Dollo <- matrix(c(0, 0, 1, 0), nrow = 2, ncol = 2 , byrow = TRUE) #customized model suppressing  web re-gain
fit_Dollo <- fitDiscrete(p, web, model = Dollo)  
aicc<-setNames(c(fit_ER$opt$aicc, fit_ARD$opt$aicc, fit_Dollo$opt$aicc), c("ER","ARD","Dollo"))
aic.w(aicc)
# perform ACE
mtree <- make.simmap(p, web, model = "ER", nsim = 100)
densityMap(mtree, lwd = 1, outline = TRUE, fsize =c(0.5,0.7), ftype = "i")

# cribellar
crib <- set_names(dat$data[, "cribellar"], rownames(dat$data))
#here we use a customized model suppressing  cribellum regain following Dollo's law based on the current consensus of spinning organ evolution
mtree <- make.simmap(p, crib, model = matrix(c(0, 1, 0, 0), nrow = 2, ncol = 2 , byrow = TRUE), nsim = 100) 
densityMap(mtree, lwd = 1, outline = TRUE, fsize =c(0.5,0.7), ftype = "i")

# above ground
strat <- set_names(dat$data[, "stratum"], rownames(dat$data))
fit_ER <- fitDiscrete(p, strat, model = "ER") 
fit_ARD <- fitDiscrete(p, strat, model = "ARD")
aicc<-setNames(c(fit_ER$opt$aicc, fit_ARD$opt$aicc), c("ER","ARD"))
aic.w(aicc)
mtree <- make.simmap(p, strat, model = "ER", nsim = 100)
densityMap(mtree, lwd = 1, outline = TRUE, fsize =c(0.5,0.7), ftype = "i")



##------ Running speed (bl/s) ------##
#prepare objects
dat <- match_dataphy(burst_speed_avg ~ 1, data = d, phy = p)
speed <- set_names(dat$data[, "burst_speed_avg"], rownames(dat$data))
web <- set_names(as.factor(dat$data[, "web"]), rownames(dat$data))
web <- to.matrix(web, levels(web))

#plot trait evolution
obj<-contMap(dat$phy, speed, plot=FALSE)
obj<-setMap(obj,colors=c("blue","grey","orange","red"))
plot(obj, lwd=c(4,5), outline=FALSE, offset=0.5, fsize=c(0.45, 0.7), legend=50, leg.txt="burst speed (bl/s)") # use ftype="off" to exclude tip labels
tiplabels(pie=web, piecol=c("white", "black"), cex=0.25, offset=0.5)
add.simmap.legend(c("cursorial spider", "web builder"), colors = c("white", "black"), shape="circle", prompt=F, fsize=0.7, x=100, y=-3)
# with ace node labels
web_ace   <- set_names(dat$data[, "web"], rownames(dat$data))
mtree <- make.simmap(dat$phy, web_ace, model = "ER", nsim = 100)
pd <- describe.simmap(mtree)
nodelabels(pie=pd$ace, piecol=c("white","black"), cex=0.25)

#compare trait means between ecological categories
# 1. model comparison for continuous trait
bm <- fitContinuous(p, speed, model = "BM")
ou <- fitContinuous(p, speed, model = "OU")
eb <- fitContinuous(p, speed, model = "EB")
lambda <- fitContinuous(p, speed, model = "lambda")
aicc <- setNames(c(bm$opt$aicc, ou$opt$aicc, eb$opt$aicc, lambda$opt$aicc), c("BM","OU","EB","lambda"))
aic.w(aicc) # returns lambda as best fitting model

# 2. phylogenetic linear regression
mod1 <- phylolm(burst_speed_avg ~ web, dat$data, dat$phy, model = "lambda")
summary(mod1)

mod2 <- phylolm(burst_speed_avg ~ cribellar, dat$data, dat$phy, model = "lambda")
summary(mod2)

mod3 <- phylolm(burst_speed_avg ~ stratum, dat$data, dat$phy, model = "lambda")
summary(mod3)

#violin plot
dev.off()
vioplot(burst_speed_avg ~ web, dat$data, col="grey90", rectCol="white", plotCentre="line", fsize=1)
stripchart(burst_speed_avg ~ web, dat$data, method = "jitter", col = "black", vertical = TRUE, pch = 19, cex = 0.5, add = TRUE)

vioplot(burst_speed_avg ~ cribellar, dat$data, col="grey90", rectCol="white", plotCentre="line", fsize=1)
stripchart(burst_speed_avg ~ cribellar, dat$data, method = "jitter", col = "black", vertical = TRUE, pch = 19, cex = 0.5, add = TRUE)

vioplot(burst_speed_avg ~ stratum, dat$data, col="grey90", rectCol="white", plotCentre="line", fsize=1)
stripchart(burst_speed_avg ~ stratum, dat$data, method = "jitter", col = "black", vertical = TRUE, pch = 19, cex = 0.5, add = TRUE)

# plot with effect size 
# set plot colours 
cols <- c(rgb(149,	57,	140, max = 255),rgb(219, 101, 23, max = 255))

## calculate effect size 
set.seed(1)
d1 <- DurgaDiff(burst_speed_avg ~ web, dat$data, contrasts = "cursorial - webbuilder", R = 5000)
d1

d2 <- DurgaDiff(burst_speed_avg ~ cribellar, dat$data, R = 5000)
d2

d3 <- DurgaDiff(burst_speed_avg ~ stratum, dat$data, R = 5000)
d3

## plot effect size 
dev.off()
par(mgp = c(2.5,1,0))
plot1 <- DurgaPlot(d1, 
                contrasts = "cursorial - webbuilder",
                box = cols,
                box.fill = DurgaTransparent(cols,0.6),
                box.params = list(boxwex = 0.2),
                violin = cols,
                violin.shape =  "full",
                violin.fill = DurgaTransparent(cols,0.8),
                points = DurgaTransparent(cols,0.4),
                points.params = list(cex = 1.0),
                las = 1,
                ef.size.params = list(las = 1),
                ef.size.pch = 15,
                left.ylab = "Burst speed avg (bl/s)",
                ef.size.label = "Mean difference (bl/s)")

plot2 <- DurgaPlot(d2, 
                box = cols,
                box.fill = DurgaTransparent(cols,0.6),
                box.params = list(boxwex = 0.2),
                violin = cols,
                violin.shape =  "full",
                violin.fill = DurgaTransparent(cols,0.8),
                points = DurgaTransparent(cols,0.4),
                points.params = list(cex = 1.0),
                las = 1,
                ef.size.params = list(las = 1),
                ef.size.pch = 15,
                left.ylab = "Burst speed avg (bl/s)",
                ef.size.label = "Mean difference (bl/s)")

plot3 <- DurgaPlot(d3, 
                box = cols,
                box.fill = DurgaTransparent(cols,0.6),
                box.params = list(boxwex = 0.2),
                violin = cols,
                violin.shape =  "full",
                violin.fill = DurgaTransparent(cols,0.8),
                points = DurgaTransparent(cols,0.4),
                points.params = list(cex = 1.0),
                las = 1,
                ef.size.params = list(las = 1),
                ef.size.pch = 15,
                left.ylab = "Burst speed avg (bl/s)",
                ef.size.label = "Mean difference (bl/s)")


#relationship between running speed and leg length
mod4 <- phylolm(burst_speed_avg ~ rel_leg_length, dat$data, dat$phy, model = "lambda")
summary(mod4)

plot <- ggplot(dat$data, aes(x = rel_leg_length, y = burst_speed_avg)) + geom_point()
plot
plot + coord_trans(x="log10",y="log10")

#relationship between running speed and spination
mod4 <- phylolm(burst_speed_avg ~ spination, dat$data, dat$phy, model = "lambda")
summary(mod4)

plot <- ggplot(dat$data, aes(x = spination, y = burst_speed_avg)) + geom_point()
plot


##------ Spination ------##
#prepare objects
dat <- match_dataphy(spination ~ 1, data = d, phy = p)
spine   <- set_names(dat$data[, "spination"], rownames(dat$data))
web   <- set_names(as.factor(dat$data[, "web"]), rownames(dat$data))
web <- to.matrix(web, levels(web))

#plot trait evolution
obj<-contMap(dat$phy, spine, plot=FALSE)
obj<-setMap(obj,colors=c("blue","grey","orange","red"))
plot(obj, lwd=c(4,5), outline=FALSE, offset=0.5, fsize=c(0.45, 0.7), legend=50, leg.txt="spination") # use ftype="off" to exclude tip labels; direction="leftwards" for left orientation
tiplabels(pie=web, piecol=c("white", "black"), cex=0.25, offset=0.5)
add.simmap.legend(c("cursorial spider", "web builder"), colors = c("white", "black"), shape="circle", prompt=F, fsize=0.7, x=100, y=-3)
# with ace node labels
web_ace   <- set_names(dat$data[, "web"], rownames(dat$data))
mtree <- make.simmap(dat$phy, web_ace, model = "ER", nsim = 100)
pd <- describe.simmap(mtree)
nodelabels(pie=pd$ace, piecol=c("white","black"), cex=0.25)

#plot OU-like trait evolution fixing node states to OU-based ACE (approach by Liam Revell)
fit<-fitContinuous(dat$phy, spine ,model="OU") #to retrieve model parameters
ou<-vector()
N<-length(dat$phy$tip) 
M<-dat$phy$Nnode
outree<-rescale(dat$phy,model="OU",fit$opt$alpha) #alpha-transforms the tree
for(i in 1:M+N){
  ou[i-N]<-ace(spine,multi2di(root(outree,node=i)),
               method="pic")$ace[1]
  names(ou)[i-N]<-i
} #to create vector of estimated ancestral states for each node by rerooting the alpha-transformed tree in each node
obj<-contMap(dat$phy, spine, method="user", anc.states=ou, plot=FALSE)
obj<-setMap(obj,colors=c("blue","grey","orange","red"))
plot(obj, lwd=c(4,5), outline=FALSE, offset=0.5, fsize=c(0.45, 0.7), legend=50, leg.txt="spination")
tiplabels(pie=web, piecol=c("white", "black"), cex=0.25, offset=0.5)
add.simmap.legend(c("cursorial spider", "web builder"), colors = c("white", "black"), shape="circle", prompt=F, fsize=0.7, x=100, y=-3)
nodelabels(pie=pd$ace, piecol=c("white","black"), cex=0.25)

#compare trait means between ecological categories
# 1. model comparison for continuous trait
bm <- fitContinuous(dat$phy, spine, model = "BM")
ou <- fitContinuous(dat$phy, spine, model = "OU")
eb <- fitContinuous(dat$phy, spine, model = "EB")
lambda <- fitContinuous(dat$phy, spine, model = "lambda")
aicc <- setNames(c(bm$opt$aicc, ou$opt$aicc, eb$opt$aicc, lambda$opt$aicc), c("BM","OU","EB","lambda"))
aic.w(aicc) # returns OU as best fitting model

# 2. phylogenetic linear regression
mod1 <- phylolm(spination ~ web, dat$data, dat$phy, model = "OUrandomRoot")
summary(mod1)

mod2 <- phylolm(spination ~ cribellar, dat$data, dat$phy, model = "OUrandomRoot")
summary(mod2)

mod3 <- phylolm(spination ~ stratum, dat$data, dat$phy, model = "OUrandomRoot")
summary(mod3)

dev.off()
vioplot(spination ~ web, dat$data, col="grey90", rectCol="white", plotCentre="line", fsize=1)
stripchart(spination ~ web, dat$data, method = "jitter", col = "black", vertical = TRUE, pch = 19, cex = 0.5, add = TRUE)

## plot with effect szie 
#calculate effect size
d4 <- DurgaDiff(spination ~ web, dat$data, contrasts = "cursorial - webbuilder", R = 5000)
d4
d5 <- DurgaDiff(spination ~ cribellar, contrasts = "ecribellar - cribellar", dat$data, R = 5000)
d5
d6 <- DurgaDiff(spination ~ stratum, contrasts = "ground - aboveground", dat$data, R = 5000)
d6

##plot effect size
dev.off()
par(mgp = c(2.5,1,0))
plot4 <- DurgaPlot(d4, 
                contrasts = "cursorial - webbuilder",
                box = cols,
                box.fill = DurgaTransparent(cols,0.6),
                box.params = list(boxwex = 0.2),
                violin = cols,
                #violin.width = 0.3, please 
                violin.shape =  "full",
                violin.fill = DurgaTransparent(cols,0.8),
                points = DurgaTransparent(cols,0.4),
                points.params = list(cex = 1.0),
                ef.size = T,
                las = 1,
                ef.size.params = list(las = 1),
                ef.size.pch = 15,
                left.ylab = "spination index avg (mm/mm)",
                ef.size.label = "Mean difference (mm/mm)")

plot5 <- DurgaPlot(d5, 
                box = cols,
                box.fill = DurgaTransparent(cols,0.6),
                box.params = list(boxwex = 0.2),
                violin = cols,
                #violin.width = 0.3, please 
                violin.shape =  "full",
                violin.fill = DurgaTransparent(cols,0.8),
                points = DurgaTransparent(cols,0.4),
                points.params = list(cex = 1.0),
                ef.size = T,
                las = 1,
                ef.size.params = list(las = 1),
                ef.size.pch = 15,
                left.ylab = "spination index avg (mm/mm)",
                ef.size.label = "Mean difference (mm/mm)")

plot6 <- DurgaPlot(d6, 
                box = cols,
                box.fill = DurgaTransparent(cols,0.6),
                box.params = list(boxwex = 0.2),
                violin = cols,
                #violin.width = 0.3, please 
                violin.shape =  "full",
                violin.fill = DurgaTransparent(cols,0.8),
                points = DurgaTransparent(cols,0.4),
                points.params = list(cex = 1.0),
                ef.size = T,
                las = 1,
                ef.size.params = list(las = 1),
                ef.size.pch = 15,
                left.ylab = "spination index avg (mm/mm)",
                ef.size.label = "Mean difference (mm/mm)")


##------ body length (mm) ------##
#prepare objects
dat <- match_dataphy(f_body_length ~ 1, data = d, phy = p)
bl   <- set_names(dat$data[, "f_body_length"], rownames(dat$data))
web   <- set_names(as.factor(dat$data[, "web"]), rownames(dat$data))
web <- to.matrix(web, levels(web))

#plot trait evolution
obj<-contMap(dat$phy, bl, plot=FALSE)
obj<-setMap(obj,colors=c("blue","grey","orange","red"))
plot(obj, lwd=c(4,5), outline=FALSE, offset=0.7, fsize=c(0.5, 0.7), leg.txt="body length (mm)") # use ftype="off" to exclude tip labels
tiplabels(pie=web, piecol=c("white", "black"), cex=0.25, offset=0.01)
add.simmap.legend(c("cursorial spider", "web builder"), colors = c("white", "black"), shape="circle", prompt=F, fsize=0.7, x=100, y=-3)

# relationship between body length and web
dev.off()
vioplot(f_body_length ~ stratum, dat$data, col="grey90", rectCol="white", plotCentre="line", fsize=1)
stripchart(f_body_length ~ stratum, dat$data, method = "jitter", col = "black", vertical = TRUE, pch = 19, add = TRUE)